import { Controller, Get, Post, Body, Patch, Param, Delete, ParseIntPipe, UseGuards, HttpCode, Req, NotFoundException, Query } from '@nestjs/common'
import { GeolocationsService } from './geolocations.service'
import { CreateGeolocationDto } from './dto/create-geolocation.dto'
import { AuthenticatedRequest, JwtAuthGuard } from 'src/auth/jwt-auth.guard'
import { ApiBearerAuth, ApiCreatedResponse, ApiForbiddenResponse, ApiNoContentResponse, ApiNotFoundResponse, ApiOkResponse, ApiUnauthorizedResponse } from '@nestjs/swagger'
import { GeolocationEntity } from './entities/geolocation.entity'
import { ApiOkResponsePaginated, paginateOutput } from 'src/common/utils/pagination.utils'
import { QueryPaginationDto } from 'src/common/dto/pagination-query.dto'
import { PaginateOutputDto } from 'src/common/dto/pagination-output.dto'

@Controller('geolocations')
export class GeolocationsController {
  constructor(private readonly geolocationsService: GeolocationsService) {}

  @Post()
  @HttpCode(201)
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiCreatedResponse({ type: GeolocationEntity, description: "Objet créé avec succès" })
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  @ApiForbiddenResponse({ description: "Permission refusée" })
  @ApiNotFoundResponse({ description: "Object non trouvé" })
  async create(@Req() request: AuthenticatedRequest, @Body() createGeolocationDto: CreateGeolocationDto): Promise<GeolocationEntity> {
    const user = request.user
    const geolocation = await this.geolocationsService.create(user, createGeolocationDto)
    return new GeolocationEntity(geolocation)
  }

  @Get()
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiOkResponsePaginated(GeolocationEntity)
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  @ApiForbiddenResponse({ description: "Permission refusée" })
  @ApiNotFoundResponse({ description: "Objet non trouvé" })
  async findAll(@Query() queryPagination?: QueryPaginationDto): Promise<PaginateOutputDto<GeolocationEntity>> {
    const [geolocations, total] = await this.geolocationsService.findAll(queryPagination)
    return paginateOutput<GeolocationEntity>(geolocations.map(geolocation => new GeolocationEntity(geolocation)), total, queryPagination)
  }

  @Get(':id')
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiOkResponse({ type: GeolocationEntity })
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  @ApiForbiddenResponse({ description: "Permission refusée" })
  @ApiNotFoundResponse({ description: "Objet non trouvé" })
  async findOne(@Param('id', ParseIntPipe) id: number): Promise<GeolocationEntity> {
    const geolocation = await this.geolocationsService.findOne(+id)
    if (!geolocation)
      throw new NotFoundException(`Géolocalisation inexistante avec l'identifiant ${id}`)
    return new GeolocationEntity(geolocation)
  }

  @Delete(':id')
  @HttpCode(204)
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth()
  @ApiNoContentResponse({ description: "Objet supprimé avec succès" })
  @ApiUnauthorizedResponse({ description: "Non authentifié⋅e" })
  @ApiForbiddenResponse({ description: "Permission refusée" })
  @ApiNotFoundResponse({ description: "Objet non trouvé" })
  async remove(@Param('id', ParseIntPipe) id: number): Promise<void> {
    await this.geolocationsService.remove(+id)
  }
}
